#!/usr/bin/env ruby
# KageFont library by eto 2003-0311

require 'sgl'
require 'kageserver'

#こんな感じのフォーマットになっている。
#<?xml version="1.0"?>
#<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.0//EN" "http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd">
#<svg viewBox="0 0 1000 1000">
#<path d="M 50,540 950,255 " style="fill: none; stroke: black; stroke-width: 10; stroke-linecap: round;"/>
#<path d="M 330,50 330,900 M 330,900 Q 330,950 380,950 M 380,950 840,950 M 840,950 Q 890,950 915,850 " style="fill: none; stroke: black; stroke-width: 10; stroke-linecap: round;"/>
#</svg>

module StrokeFont
  class KageParser #======================================================================
    def self.parse(svg)
      @strokes = Strokes.new
      lines = svg.split(/\n/)
      lines.each {|line|
	next unless line =~ /^<path/
	if line =~ /d=\"([a-zA-Z0-9, ]+)\"/
	  KageParser.parse_path($1)
	end
      }
      return @strokes
    end
    def self.parse_path(str)
      #M 50,540 950,255 
      #M 330,50 330,900 M 330,900 Q 330,950 380,950 M 380,950 840,950 M 840,950 Q 890,950 915,850 
      px, py = -1, -1
      str.split.each {|par|
	next if par.length == 1 #一文字はとばします。本来はちゃんとベジェを展開する。
	if par =~ /[,0-9]+/
	  sx, sy = par.split(/,/)
	  x, y = sx.to_i, (1000 - sy.to_i)
	  if px != -1
	    @strokes.add_line(px, py, x, y)
	  end
	  px, py = x, y
	end
      }
    end
  end

  class KageGlyph #====================================================================== てなかんじ
    def initialize(code, svg)
      @code = code
      @svg = svg
      @strokes = nil
    end
    attr_reader :strokes
    def parse
      return if @strokes
      @strokes = KageParser.parse(@svg)
    end
    def init
      parse if @strokes.nil?
    end
  end

  class KageFont #======================================================================
    def initialize
      @server = KageServer.instance
      @glyphs = []
      @cache_list = @server.list_cache
      @rend = nil
      @rend = StrokesRenderer.new
      @rend.hsv = [13, 100, 100]
    end
    attr_reader :cache_list
    def get(code)
      return @glyphs[code] if @glyphs[code]
      svg = @server.get(code)
      return nil if svg.nil?
      @glyphs[code] = KageGlyph.new(code, svg)
    end
    def init(code)
      glyph = get(code)
      return if glyph.nil?
      glyph.init
      glyph.parse
      @rend.set_strokes(glyph.strokes)
    end
    def draw(code)
      glyph = get(code)
      return if glyph.nil?
      @rend.draw
    end
    def print(code)
      char = Character.new(@code)
      printf("[%s][%04x]\n", char.nil? ? "nil" : char.map_sjis, code)
    end
  end

end

#----------------------------------------------------------------------end.
